@@ -62,7 +62,8 @@ html_theme = "pydata_sphinx_theme"
 # Add any paths that contain custom static files (such as style sheets) here,
 # relative to this directory. They are copied after the builtin static files,
 # so a file named "default.css" will overwrite the builtin "default.css".
-# html_static_path = ["_static"]
+html_static_path = ["_static"]
+html_css_files = ["css/custom.css"]  # relative to html_static_path
 
 html_logo = "../img/polars_logo.png"
 autosummary_generate = True
