@@ -24,12 +24,6 @@ use std::borrow::Cow;
 
 macro_rules! impl_dyn_series {
     ($ca: ident) => {
-        impl IntoSeries for $ca {
-            fn into_series(self) -> Series {
-                Series(Arc::new(SeriesWrap(self)))
-            }
-        }
-
         impl private::PrivateSeries for SeriesWrap<$ca> {
             fn _field(&self) -> Cow<Field> {
                 Cow::Borrowed(self.0.ref_field())
@@ -52,8 +46,8 @@ macro_rules! impl_dyn_series {
                 self.0.cummin(reverse).into_series()
             }
 
-            fn _set_sorted(&mut self, reverse: bool) {
-                self.0.set_sorted(reverse)
+            fn _set_sorted(&mut self, is_sorted: IsSorted) {
+                self.0.set_sorted2(is_sorted)
             }
 
             unsafe fn equal_element(
@@ -279,7 +273,7 @@ macro_rules! impl_dyn_series {
                 let mut out = ChunkTake::take_unchecked(&self.0, (&*idx).into());
 
                 if self.0.is_sorted() && (idx.is_sorted() || idx.is_sorted_reverse()) {
-                    out.set_sorted(idx.is_sorted_reverse())
+                    out.set_sorted2(idx.is_sorted2())
                 }
 
                 Ok(out.into_series())
