@@ -1,4 +1,5 @@
 mod list;
+mod strings;
 #[cfg(feature = "to_dummies")]
 mod to_dummies;
 
@@ -11,3 +12,4 @@ use polars_core::prelude::*;
 pub use to_dummies::*;
 
 pub use list::*;
+pub use strings::*;
