@@ -16,6 +16,13 @@ pub trait DateMethods {
     /// Returns the year number in the calendar date.
     fn year(&self) -> Int32Chunked;
 
+    /// Extract month from underlying NaiveDateTime representation.
+    /// Quarters range from 1 to 4.
+    fn quarter(&self) -> UInt32Chunked {
+        let months = self.month();
+        months_to_quarters(months)
+    }
+
     /// Extract month from underlying NaiveDateTime representation.
     /// Returns the month number starting from 1.
     ///
