@@ -1,6 +1,5 @@
 pub(super) mod no_nulls;
 use crate::prelude::*;
-use arrow::array::ArrayRef;
 use arrow::array::PrimitiveArray;
 use arrow::types::NativeType;
 use polars_arrow::data_types::IsFloat;
@@ -11,4 +10,3 @@ use polars_core::export::num::{Bounded, Float, NumCast, One};
 use polars_core::prelude::*;
 use std::fmt::Debug;
 use std::ops::{AddAssign, Div, Mul, Sub, SubAssign};
-use std::sync::Arc;
