@@ -1,35 +1,41 @@
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Dict,
-    Mapping,
-    Optional,
-    Sequence,
-    Union,
-    overload,
-)
+from __future__ import annotations
 
-import numpy as np
+import sys
+import warnings
+from typing import Any, Mapping, Sequence, overload
 
 from polars.internals import DataFrame, Series
 
-if TYPE_CHECKING:  # pragma: no cover
-    import pandas as pd
+try:
+    import numpy as np
+
+    _NUMPY_AVAILABLE = True
+except ImportError:
+    _NUMPY_AVAILABLE = False
+
+try:
     import pyarrow as pa
 
     _PYARROW_AVAILABLE = True
-else:
-    try:
-        import pyarrow as pa
+except ImportError:
+    _PYARROW_AVAILABLE = False
 
-        _PYARROW_AVAILABLE = True
-    except ImportError:  # pragma: no cover
-        _PYARROW_AVAILABLE = False
+try:
+    import pandas as pd
+
+    _PANDAS_AVAILABLE = True
+except ImportError:
+    _PANDAS_AVAILABLE = False
+
+if sys.version_info >= (3, 8):
+    from typing import Literal
+else:
+    from typing_extensions import Literal  # pragma: no cover
 
 
 def from_dict(
-    data: Mapping[str, Union[Sequence, Mapping]],
-    columns: Optional[Sequence[str]] = None,
+    data: Mapping[str, Sequence | Mapping],
+    columns: Sequence[str] | None = None,
 ) -> DataFrame:
     """
     Construct a DataFrame from a dictionary of sequences.
@@ -49,7 +55,6 @@ def from_dict(
 
     Examples
     --------
-
     >>> data = {"a": [1, 2], "b": [3, 4]}
     >>> df = pl.from_dict(data)
     >>> df
@@ -68,10 +73,50 @@ def from_dict(
     return DataFrame._from_dict(data=data, columns=columns)  # type: ignore
 
 
+def from_dicts(
+    dicts: Sequence[dict[str, Any]], infer_schema_length: int | None = 50
+) -> DataFrame:
+    """
+    Construct a DataFrame from a sequence of dictionaries.
+
+    Parameters
+    ----------
+    dicts
+        Sequence with dictionaries mapping column name to value
+    infer_schema_length
+        How many dictionaries/rows to scan to determine the data types
+        if set to `None` all rows are scanned. This will be slow.
+
+    Returns
+    -------
+    DataFrame
+
+    Examples
+    --------
+    >>> data = [{"a": 1, "b": 4}, {"a": 2, "b": 5}, {"a": 3, "b": 6}]
+    >>> df = pl.from_dicts(data)
+    >>> df
+    shape: (3, 2)
+    ┌─────┬─────┐
+    │ a   ┆ b   │
+    │ --- ┆ --- │
+    │ i64 ┆ i64 │
+    ╞═════╪═════╡
+    │ 1   ┆ 4   │
+    ├╌╌╌╌╌┼╌╌╌╌╌┤
+    │ 2   ┆ 5   │
+    ├╌╌╌╌╌┼╌╌╌╌╌┤
+    │ 3   ┆ 6   │
+    └─────┴─────┘
+
+    """
+    return DataFrame._from_dicts(dicts, infer_schema_length)
+
+
 def from_records(
-    data: Union[np.ndarray, Sequence[Sequence[Any]]],
-    columns: Optional[Sequence[str]] = None,
-    orient: Optional[str] = None,
+    data: Sequence[Sequence[Any]],
+    columns: Sequence[str] | None = None,
+    orient: Literal["col", "row"] | None = None,
 ) -> DataFrame:
     """
     Construct a DataFrame from a numpy ndarray or sequence of sequences.
@@ -96,51 +141,8 @@ def from_records(
 
     Examples
     --------
-
     >>> data = [[1, 2, 3], [4, 5, 6]]
     >>> df = pl.from_records(data, columns=["a", "b"])
-    >>> df
-        shape: (3, 2)
-    ┌─────┬─────┐
-    │ a   ┆ b   │
-    │ --- ┆ --- │
-    │ i64 ┆ i64 │
-    ╞═════╪═════╡
-    │ 1   ┆ 4   │
-    ├╌╌╌╌╌┼╌╌╌╌╌┤
-    │ 2   ┆ 5   │
-    ├╌╌╌╌╌┼╌╌╌╌╌┤
-    │ 3   ┆ 6   │
-    └─────┴─────┘
-
-    """
-    return DataFrame._from_records(data, columns=columns, orient=orient)
-
-
-def from_dicts(
-    dicts: Sequence[Dict[str, Any]],
-    infer_schema_length: Optional[int] = 50,
-) -> DataFrame:
-    """
-    Construct a DataFrame from a sequence of dictionaries.
-
-    Parameters
-    ----------
-    dicts
-        Sequence with dictionaries mapping column name to value
-    infer_schema_length
-        How many dictionaries/rows to scan to determine the data types
-        if set to `None` all rows are scanned. This will be slow.
-
-    Returns
-    -------
-    DataFrame
-
-    Examples
-    --------
-
-    >>> data = [{"a": 1, "b": 4}, {"a": 2, "b": 5}, {"a": 3, "b": 6}]
-    >>> df = pl.from_dicts(data)
     >>> df
     shape: (3, 2)
     ┌─────┬─────┐
@@ -156,13 +158,72 @@ def from_dicts(
     └─────┴─────┘
 
     """
-    return DataFrame._from_dicts(dicts, infer_schema_length)
+    if _NUMPY_AVAILABLE and isinstance(data, np.ndarray):
+        warnings.warn(
+            "using `from_records` with a numpy ndarray is deprecated, "
+            "use `from_numpy` instead",
+            DeprecationWarning,
+        )
+        return DataFrame._from_numpy(data, columns=columns, orient=orient)
+    else:
+        return DataFrame._from_records(data, columns=columns, orient=orient)
+
+
+def from_numpy(
+    data: np.ndarray,
+    columns: Sequence[str] | None = None,
+    orient: Literal["col", "row"] | None = None,
+) -> DataFrame:
+    """
+    Construct a DataFrame from a numpy ndarray.
+
+    Note that this is slower than creating from columnar memory.
+
+    Parameters
+    ----------
+    data : numpy ndarray
+        Two-dimensional data represented as a numpy ndarray.
+    columns : Sequence of str, default None
+        Column labels to use for resulting DataFrame. Must match data dimensions.
+        If not specified, columns will be named `column_0`, `column_1`, etc.
+    orient : {'col', 'row'}, default None
+        Whether to interpret two-dimensional data as columns or as rows. If None,
+        the orientation is inferred by matching the columns and data dimensions. If
+        this does not yield conclusive results, column orientation is used.
+
+    Returns
+    -------
+    DataFrame
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> data = np.array([[1, 2, 3], [4, 5, 6]])
+    >>> df = pl.from_numpy(data, columns=["a", "b"], orient="col")
+    >>> df
+    shape: (3, 2)
+    ┌─────┬─────┐
+    │ a   ┆ b   │
+    │ --- ┆ --- │
+    │ i64 ┆ i64 │
+    ╞═════╪═════╡
+    │ 1   ┆ 4   │
+    ├╌╌╌╌╌┼╌╌╌╌╌┤
+    │ 2   ┆ 5   │
+    ├╌╌╌╌╌┼╌╌╌╌╌┤
+    │ 3   ┆ 6   │
+    └─────┴─────┘
+
+    """
+    if not _NUMPY_AVAILABLE:
+        raise ImportError("'numpy' is required when using from_numpy().")
+    return DataFrame._from_numpy(data, columns=columns, orient=orient)
 
 
 # Note that we cannot overload because pyarrow has no stubs :(
 def from_arrow(
-    a: Union["pa.Table", "pa.Array", "pa.ChunkedArray"], rechunk: bool = True
-) -> Union[DataFrame, Series]:
+    a: pa.Table | pa.Array | pa.ChunkedArray, rechunk: bool = True
+) -> DataFrame | Series:
     """
     Create a DataFrame or Series from an Arrow Table or Array.
 
@@ -217,9 +278,7 @@ def from_arrow(
 
     """
     if not _PYARROW_AVAILABLE:
-        raise ImportError(
-            "'pyarrow' is required when using from_arrow()."
-        )  # pragma: no cover
+        raise ImportError("'pyarrow' is required when using from_arrow().")
     if isinstance(a, pa.Table):
         return DataFrame._from_arrow(a, rechunk=rechunk)
     elif isinstance(a, (pa.Array, pa.ChunkedArray)):
@@ -230,7 +289,7 @@ def from_arrow(
 
 @overload
 def from_pandas(
-    df: "pd.DataFrame",
+    df: pd.DataFrame,
     rechunk: bool = True,
     nan_to_none: bool = True,
 ) -> DataFrame:
@@ -239,7 +298,7 @@ def from_pandas(
 
 @overload
 def from_pandas(
-    df: Union["pd.Series", "pd.DatetimeIndex"],
+    df: pd.Series | pd.DatetimeIndex,
     rechunk: bool = True,
     nan_to_none: bool = True,
 ) -> Series:
@@ -247,10 +306,10 @@ def from_pandas(
 
 
 def from_pandas(
-    df: Union["pd.DataFrame", "pd.Series", "pd.DatetimeIndex"],
+    df: pd.DataFrame | pd.Series | pd.DatetimeIndex,
     rechunk: bool = True,
     nan_to_none: bool = True,
-) -> Union[DataFrame, Series]:
+) -> DataFrame | Series:
     """
     Construct a Polars DataFrame or Series from a pandas DataFrame or Series.
 
@@ -304,14 +363,9 @@ def from_pandas(
 
     """
     if not _PYARROW_AVAILABLE:
-        raise ImportError(  # pragma: no cover
-            "'pyarrow' is required when using from_pandas()."
-        )
-
-    try:
-        import pandas as pd
-    except ImportError as e:  # pragma: no cover
-        raise ImportError("'pandas' is required when using from_pandas().") from e
+        raise ImportError("'pyarrow' is required when using from_pandas().")
+    if not _PANDAS_AVAILABLE:
+        raise ImportError("'pandas' is required when using from_pandas().")
 
     if isinstance(df, (pd.Series, pd.DatetimeIndex)):
         return Series._from_pandas("", df, nan_to_none=nan_to_none)
