@@ -1,17 +1,19 @@
+from __future__ import annotations
+
 import sys
 from datetime import date, datetime, timedelta
 from inspect import isclass
-from typing import Any, Callable, List, Optional, Sequence, Type, Union, cast, overload
-
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal  # pragma: no cover
-
-import numpy as np
+from typing import Any, Callable, Sequence, cast, overload
 
 from polars import internals as pli
-from polars.datatypes import DataType, Date, Datetime, Duration, py_type_to_dtype
+from polars.datatypes import (
+    DataType,
+    Date,
+    Datetime,
+    Duration,
+    PolarsDataType,
+    py_type_to_dtype,
+)
 from polars.utils import (
     _datetime_to_pl_timestamp,
     _timedelta_to_pl_timedelta,
@@ -21,6 +23,7 @@ from polars.utils import (
 
 try:
     from polars.polars import arange as pyarange
+    from polars.polars import arg_where as py_arg_where
     from polars.polars import argsort_by as pyargsort_by
     from polars.polars import as_struct as _as_struct
     from polars.polars import binary_function as pybinary_function
@@ -45,13 +48,25 @@ try:
     from polars.polars import spearman_rank_corr as pyspearman_rank_corr
 
     _DOCUMENTING = False
-except ImportError:  # pragma: no cover
+except ImportError:
     _DOCUMENTING = True
 
+try:
+    import numpy as np
+
+    _NUMPY_AVAILABLE = True
+except ImportError:
+    _NUMPY_AVAILABLE = False
+
+if sys.version_info >= (3, 8):
+    from typing import Literal
+else:
+    from typing_extensions import Literal
+
 
 def col(
-    name: Union[str, List[str], List[Type[DataType]], "pli.Series", Type[DataType]]
-) -> "pli.Expr":
+    name: (str | list[str] | Sequence[PolarsDataType] | pli.Series | PolarsDataType),
+) -> pli.Expr:
     """
     A column in a DataFrame.
     Can be used to select:
@@ -67,7 +82,6 @@ def col(
 
     Examples
     --------
-
     >>> df = pl.DataFrame(
     ...     {
     ...         "ham": [1, 2, 3],
@@ -151,23 +165,29 @@ def col(
     if isclass(name) and issubclass(cast(type, name), DataType):
         name = [cast(type, name)]
 
+    if isinstance(name, DataType):
+        return pli.wrap_expr(_dtype_cols([name]))
+
     if isinstance(name, list):
         if len(name) == 0 or isinstance(name[0], str):
             return pli.wrap_expr(pycols(name))
-        elif isclass(name[0]) and issubclass(name[0], DataType):
+        elif (
+            isclass(name[0])
+            and issubclass(name[0], DataType)
+            or isinstance(name[0], DataType)
+        ):
             return pli.wrap_expr(_dtype_cols(name))
         else:
             raise ValueError("did expect argument of List[str] or List[DataType]")
     return pli.wrap_expr(pycol(name))
 
 
-def element() -> "pli.Expr":
+def element() -> pli.Expr:
     """
     Alias for an element in evaluated in an `eval` expression
 
     Examples
     --------
-
     A horizontal rank computation by taking the elements of a list
 
     >>> df = pl.DataFrame({"a": [1, 8, 3], "b": [4, 5, 2]})
@@ -178,7 +198,7 @@ def element() -> "pli.Expr":
     ┌─────┬─────┬────────────┐
     │ a   ┆ b   ┆ rank       │
     │ --- ┆ --- ┆ ---        │
-    │ i64 ┆ i64 ┆ list [f32] │
+    │ i64 ┆ i64 ┆ list[f32]  │
     ╞═════╪═════╪════════════╡
     │ 1   ┆ 4   ┆ [1.0, 2.0] │
     ├╌╌╌╌╌┼╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌┤
@@ -212,21 +232,21 @@ def element() -> "pli.Expr":
 
 
 @overload
-def count(column: str) -> "pli.Expr":
+def count(column: str) -> pli.Expr:
     ...
 
 
 @overload
-def count(column: "pli.Series") -> int:
+def count(column: pli.Series) -> int:
     ...
 
 
 @overload
-def count(column: None = None) -> "pli.Expr":
+def count(column: None = None) -> pli.Expr:
     ...
 
 
-def count(column: Optional[Union[str, "pli.Series"]] = None) -> Union["pli.Expr", int]:
+def count(column: str | pli.Series | None = None) -> pli.Expr | int:
     """
     Count the number of values in this column/context.
 
@@ -246,7 +266,7 @@ def count(column: Optional[Union[str, "pli.Series"]] = None) -> Union["pli.Expr"
     return col(column).count()
 
 
-def to_list(name: str) -> "pli.Expr":
+def to_list(name: str) -> pli.Expr:
     """
     Aggregate to list.
 
@@ -256,16 +276,16 @@ def to_list(name: str) -> "pli.Expr":
 
 
 @overload
-def std(column: str) -> "pli.Expr":
+def std(column: str) -> pli.Expr:
     ...
 
 
 @overload
-def std(column: "pli.Series") -> Optional[float]:
+def std(column: pli.Series) -> float | None:
     ...
 
 
-def std(column: Union[str, "pli.Series"]) -> Union["pli.Expr", Optional[float]]:
+def std(column: str | pli.Series) -> pli.Expr | float | None:
     """
     Get the standard deviation.
     """
@@ -275,16 +295,16 @@ def std(column: Union[str, "pli.Series"]) -> Union["pli.Expr", Optional[float]]:
 
 
 @overload
-def var(column: str) -> "pli.Expr":
+def var(column: str) -> pli.Expr:
     ...
 
 
 @overload
-def var(column: "pli.Series") -> Optional[float]:
+def var(column: pli.Series) -> float | None:
     ...
 
 
-def var(column: Union[str, "pli.Series"]) -> Union["pli.Expr", Optional[float]]:
+def var(column: str | pli.Series) -> pli.Expr | float | None:
     """
     Get the variance.
     """
@@ -294,18 +314,16 @@ def var(column: Union[str, "pli.Series"]) -> Union["pli.Expr", Optional[float]]:
 
 
 @overload
-def max(column: Union[str, List[Union["pli.Expr", str]]]) -> "pli.Expr":
+def max(column: str | list[pli.Expr | str]) -> pli.Expr:
     ...
 
 
 @overload
-def max(column: "pli.Series") -> Union[int, float]:
+def max(column: pli.Series) -> int | float:
     ...
 
 
-def max(
-    column: Union[str, List[Union["pli.Expr", str]], "pli.Series"]
-) -> Union["pli.Expr", Any]:
+def max(column: str | list[pli.Expr | str] | pli.Series) -> pli.Expr | Any:
     """
     Get the maximum value. Can be used horizontally or vertically.
 
@@ -327,18 +345,16 @@ def max(
 
 
 @overload
-def min(column: Union[str, List[Union["pli.Expr", str]]]) -> "pli.Expr":
+def min(column: str | list[pli.Expr | str]) -> pli.Expr:
     ...
 
 
 @overload
-def min(column: "pli.Series") -> Union[int, float]:
+def min(column: pli.Series) -> int | float:
     ...
 
 
-def min(
-    column: Union[str, List[Union["pli.Expr", str]], "pli.Series"]
-) -> Union["pli.Expr", Any]:
+def min(column: str | list[pli.Expr | str] | pli.Series) -> pli.Expr | Any:
     """
     Get the minimum value.
 
@@ -358,18 +374,16 @@ def min(
 
 
 @overload
-def sum(column: Union[str, List[Union["pli.Expr", str]]]) -> "pli.Expr":
+def sum(column: str | list[pli.Expr | str]) -> pli.Expr:
     ...
 
 
 @overload
-def sum(column: "pli.Series") -> Union[int, float]:
+def sum(column: pli.Series) -> int | float:
     ...
 
 
-def sum(
-    column: Union[str, List[Union["pli.Expr", str]], "pli.Series"]
-) -> Union["pli.Expr", Any]:
+def sum(column: str | list[pli.Expr | str] | pli.Series) -> pli.Expr | Any:
     """
     Get the sum value.
 
@@ -391,16 +405,16 @@ def sum(
 
 
 @overload
-def mean(column: str) -> "pli.Expr":
+def mean(column: str) -> pli.Expr:
     ...
 
 
 @overload
-def mean(column: "pli.Series") -> float:
+def mean(column: pli.Series) -> float:
     ...
 
 
-def mean(column: Union[str, "pli.Series"]) -> Union["pli.Expr", float]:
+def mean(column: str | pli.Series) -> pli.Expr | float:
     """
     Get the mean value.
     """
@@ -410,16 +424,16 @@ def mean(column: Union[str, "pli.Series"]) -> Union["pli.Expr", float]:
 
 
 @overload
-def avg(column: str) -> "pli.Expr":
+def avg(column: str) -> pli.Expr:
     ...
 
 
 @overload
-def avg(column: "pli.Series") -> float:
+def avg(column: pli.Series) -> float:
     ...
 
 
-def avg(column: Union[str, "pli.Series"]) -> Union["pli.Expr", float]:
+def avg(column: str | pli.Series) -> pli.Expr | float:
     """
     Alias for mean.
     """
@@ -427,16 +441,16 @@ def avg(column: Union[str, "pli.Series"]) -> Union["pli.Expr", float]:
 
 
 @overload
-def median(column: str) -> "pli.Expr":
+def median(column: str) -> pli.Expr:
     ...
 
 
 @overload
-def median(column: "pli.Series") -> Union[float, int]:
+def median(column: pli.Series) -> float | int:
     ...
 
 
-def median(column: Union[str, "pli.Series"]) -> Union["pli.Expr", float, int]:
+def median(column: str | pli.Series) -> pli.Expr | float | int:
     """
     Get the median value.
     """
@@ -446,16 +460,16 @@ def median(column: Union[str, "pli.Series"]) -> Union["pli.Expr", float, int]:
 
 
 @overload
-def n_unique(column: str) -> "pli.Expr":
+def n_unique(column: str) -> pli.Expr:
     ...
 
 
 @overload
-def n_unique(column: "pli.Series") -> int:
+def n_unique(column: pli.Series) -> int:
     ...
 
 
-def n_unique(column: Union[str, "pli.Series"]) -> Union["pli.Expr", int]:
+def n_unique(column: str | pli.Series) -> pli.Expr | int:
     """Count unique values."""
     if isinstance(column, pli.Series):
         return column.n_unique()
@@ -463,21 +477,21 @@ def n_unique(column: Union[str, "pli.Series"]) -> Union["pli.Expr", int]:
 
 
 @overload
-def first(column: str) -> "pli.Expr":
+def first(column: str) -> pli.Expr:
     ...
 
 
 @overload
-def first(column: "pli.Series") -> Any:
+def first(column: pli.Series) -> Any:
     ...
 
 
 @overload
-def first(column: None = None) -> "pli.Expr":
+def first(column: None = None) -> pli.Expr:
     ...
 
 
-def first(column: Optional[Union[str, "pli.Series"]] = None) -> Union["pli.Expr", Any]:
+def first(column: str | pli.Series | None = None) -> pli.Expr | Any:
     """
     Get the first value.
 
@@ -503,21 +517,21 @@ def first(column: Optional[Union[str, "pli.Series"]] = None) -> Union["pli.Expr"
 
 
 @overload
-def last(column: str) -> "pli.Expr":
+def last(column: str) -> pli.Expr:
     ...
 
 
 @overload
-def last(column: "pli.Series") -> Any:
+def last(column: pli.Series) -> Any:
     ...
 
 
 @overload
-def last(column: None = None) -> "pli.Expr":
+def last(column: None = None) -> pli.Expr:
     ...
 
 
-def last(column: Optional[Union[str, "pli.Series"]] = None) -> "pli.Expr":
+def last(column: str | pli.Series | None = None) -> pli.Expr:
     """
     Get the last value.
 
@@ -542,18 +556,16 @@ def last(column: Optional[Union[str, "pli.Series"]] = None) -> "pli.Expr":
 
 
 @overload
-def head(column: str, n: Optional[int]) -> "pli.Expr":
+def head(column: str, n: int | None) -> pli.Expr:
     ...
 
 
 @overload
-def head(column: "pli.Series", n: Optional[int]) -> "pli.Series":
+def head(column: pli.Series, n: int | None) -> pli.Series:
     ...
 
 
-def head(
-    column: Union[str, "pli.Series"], n: Optional[int] = None
-) -> Union["pli.Expr", "pli.Series"]:
+def head(column: str | pli.Series, n: int | None = None) -> pli.Expr | pli.Series:
     """
     Get the first n rows of an Expression.
 
@@ -570,18 +582,16 @@ def head(
 
 
 @overload
-def tail(column: str, n: Optional[int]) -> "pli.Expr":
+def tail(column: str, n: int | None) -> pli.Expr:
     ...
 
 
 @overload
-def tail(column: "pli.Series", n: Optional[int]) -> "pli.Series":
+def tail(column: pli.Series, n: int | None) -> pli.Series:
     ...
 
 
-def tail(
-    column: Union[str, "pli.Series"], n: Optional[int] = None
-) -> Union["pli.Expr", "pli.Series"]:
+def tail(column: str | pli.Series, n: int | None = None) -> pli.Expr | pli.Series:
     """
     Get the last n rows of an Expression.
 
@@ -598,11 +608,9 @@ def tail(
 
 
 def lit(
-    value: Optional[
-        Union[float, int, str, date, datetime, "pli.Series", np.ndarray, Any]
-    ],
-    dtype: Optional[Type[DataType]] = None,
-) -> "pli.Expr":
+    value: None | (float | int | str | date | datetime | pli.Series | np.ndarray | Any),
+    dtype: type[DataType] | None = None,
+) -> pli.Expr:
     """
     A literal value.
 
@@ -615,7 +623,6 @@ def lit(
 
     Examples
     --------
-
     Literal integer:
 
     >>> pl.lit(1)  # doctest: +IGNORE_RESULT
@@ -670,7 +677,7 @@ def lit(
             return e
         return e.alias(name)
 
-    if isinstance(value, np.ndarray):
+    if _NUMPY_AVAILABLE and isinstance(value, np.ndarray):
         return lit(pli.Series("", value))
 
     if dtype:
@@ -683,9 +690,9 @@ def lit(
 
 
 def spearman_rank_corr(
-    a: Union[str, "pli.Expr"],
-    b: Union[str, "pli.Expr"],
-) -> "pli.Expr":
+    a: str | pli.Expr,
+    b: str | pli.Expr,
+) -> pli.Expr:
     """
     Compute the spearman rank correlation between two columns.
 
@@ -704,9 +711,9 @@ def spearman_rank_corr(
 
 
 def pearson_corr(
-    a: Union[str, "pli.Expr"],
-    b: Union[str, "pli.Expr"],
-) -> "pli.Expr":
+    a: str | pli.Expr,
+    b: str | pli.Expr,
+) -> pli.Expr:
     """
     Compute the pearson's correlation between two columns.
 
@@ -725,9 +732,9 @@ def pearson_corr(
 
 
 def cov(
-    a: Union[str, "pli.Expr"],
-    b: Union[str, "pli.Expr"],
-) -> "pli.Expr":
+    a: str | pli.Expr,
+    b: str | pli.Expr,
+) -> pli.Expr:
     """
     Compute the covariance between two columns/ expressions.
 
@@ -746,10 +753,10 @@ def cov(
 
 
 def map(
-    exprs: Union[List[str], List["pli.Expr"]],
-    f: Callable[[List["pli.Series"]], "pli.Series"],
-    return_dtype: Optional[Type[DataType]] = None,
-) -> "pli.Expr":
+    exprs: list[str] | list[pli.Expr],
+    f: Callable[[list[pli.Series]], pli.Series],
+    return_dtype: type[DataType] | None = None,
+) -> pli.Expr:
     """
     Map a custom function over multiple columns/expressions and produce a single Series result.
 
@@ -771,19 +778,17 @@ def map(
 
 
 def apply(
-    exprs: List[Union[str, "pli.Expr"]],
-    f: Callable[[List["pli.Series"]], Union["pli.Series", Any]],
-    return_dtype: Optional[Type[DataType]] = None,
-) -> "pli.Expr":
+    exprs: list[str | pli.Expr],
+    f: Callable[[list[pli.Series]], pli.Series | Any],
+    return_dtype: type[DataType] | None = None,
+) -> pli.Expr:
     """
     Apply a custom function in a GroupBy context.
 
     Depending on the context it has the following behavior:
 
-    ## Context
-
-    * Select/Project
-        Don't do this, use `map`
+    * Select
+        Don't use apply, use `map`
     * GroupBy
         expected type `f`: Callable[[Series], Series]
         Applies a python function over each group.
@@ -806,16 +811,17 @@ def apply(
 
 
 def map_binary(
-    a: Union[str, "pli.Expr"],
-    b: Union[str, "pli.Expr"],
-    f: Callable[["pli.Series", "pli.Series"], "pli.Series"],
-    return_dtype: Optional[Type[DataType]] = None,
-) -> "pli.Expr":
+    a: str | pli.Expr,
+    b: str | pli.Expr,
+    f: Callable[[pli.Series, pli.Series], pli.Series],
+    return_dtype: type[DataType] | None = None,
+) -> pli.Expr:
     """
-     .. deprecated:: 0.10.4
-       use `map` or `apply`
     Map a custom function over two columns and produce a single Series result.
 
+    .. deprecated:: 0.10.4
+        Use :func:`map` or :func:`apply` instead.
+
     Parameters
     ----------
     a
@@ -835,10 +841,10 @@ def map_binary(
 
 
 def fold(
-    acc: "pli.Expr",
-    f: Callable[["pli.Series", "pli.Series"], "pli.Series"],
-    exprs: Union[Sequence[Union["pli.Expr", str]], "pli.Expr"],
-) -> "pli.Expr":
+    acc: pli.IntoExpr,
+    f: Callable[[pli.Series, pli.Series], pli.Series],
+    exprs: Sequence[pli.Expr | str] | pli.Expr,
+) -> pli.Expr:
     """
     Accumulate over multiple columns horizontally/ row wise with a left fold.
 
@@ -863,7 +869,7 @@ def fold(
     return pli.wrap_expr(pyfold(acc._pyexpr, f, exprs))
 
 
-def any(name: Union[str, List["pli.Expr"]]) -> "pli.Expr":
+def any(name: str | list[pli.Expr]) -> pli.Expr:
     """
     Evaluate columnwise or elementwise with a bitwise OR operation.
     """
@@ -874,63 +880,109 @@ def any(name: Union[str, List["pli.Expr"]]) -> "pli.Expr":
     return col(name).any()
 
 
-def exclude(columns: Union[str, List[str]]) -> "pli.Expr":
+def exclude(
+    columns: (
+        str
+        | list[str]
+        | DataType
+        | type[DataType]
+        | DataType
+        | Sequence[DataType | type[DataType]]
+    ),
+) -> pli.Expr:
     """
-    Exclude certain columns from a wildcard expression.
+    Exclude certain columns from a wildcard/regex selection.
 
     Syntactic sugar for:
 
-    >>> pl.col("*").exclude(columns)  # doctest: +SKIP
+    >>> pl.all().exclude(columns)  # doctest: +SKIP
 
     Parameters
     ----------
     columns
         Column(s) to exclude from selection
+        This can be:
 
+        - a column name, or multiple column names
+        - a regular expression starting with `^` and ending with `$`
+        - a dtype or multiple dtypes
 
     Examples
     --------
-
     >>> df = pl.DataFrame(
     ...     {
-    ...         "a": [1, 2, 3],
-    ...         "b": ["a", "b", None],
-    ...         "c": [None, 2, 1],
+    ...         "aa": [1, 2, 3],
+    ...         "ba": ["a", "b", None],
+    ...         "cc": [None, 2.5, 1.5],
     ...     }
     ... )
     >>> df
     shape: (3, 3)
     ┌─────┬──────┬──────┐
-    │ a   ┆ b    ┆ c    │
+    │ aa  ┆ ba   ┆ cc   │
     │ --- ┆ ---  ┆ ---  │
-    │ i64 ┆ str  ┆ i64  │
+    │ i64 ┆ str  ┆ f64  │
     ╞═════╪══════╪══════╡
     │ 1   ┆ a    ┆ null │
     ├╌╌╌╌╌┼╌╌╌╌╌╌┼╌╌╌╌╌╌┤
-    │ 2   ┆ b    ┆ 2    │
+    │ 2   ┆ b    ┆ 2.5  │
     ├╌╌╌╌╌┼╌╌╌╌╌╌┼╌╌╌╌╌╌┤
-    │ 3   ┆ null ┆ 1    │
+    │ 3   ┆ null ┆ 1.5  │
     └─────┴──────┴──────┘
 
-    >>> df.select(pl.exclude("b"))
+    Exclude by column name(s):
+
+    >>> df.select(pl.exclude("ba"))
     shape: (3, 2)
     ┌─────┬──────┐
-    │ a   ┆ c    │
+    │ aa  ┆ cc   │
     │ --- ┆ ---  │
-    │ i64 ┆ i64  │
+    │ i64 ┆ f64  │
     ╞═════╪══════╡
     │ 1   ┆ null │
     ├╌╌╌╌╌┼╌╌╌╌╌╌┤
-    │ 2   ┆ 2    │
+    │ 2   ┆ 2.5  │
     ├╌╌╌╌╌┼╌╌╌╌╌╌┤
-    │ 3   ┆ 1    │
+    │ 3   ┆ 1.5  │
     └─────┴──────┘
 
+    Exclude by regex, e.g. removing all columns whose names end with the letter "a":
+
+    >>> df.select(pl.exclude("^.*a$"))
+    shape: (3, 1)
+    ┌──────┐
+    │ cc   │
+    │ ---  │
+    │ f64  │
+    ╞══════╡
+    │ null │
+    ├╌╌╌╌╌╌┤
+    │ 2.5  │
+    ├╌╌╌╌╌╌┤
+    │ 1.5  │
+    └──────┘
+
+    Exclude by dtype(s), e.g. removing all columns of type Int64 or Float64:
+
+    >>> df.select(pl.exclude([pl.Int64, pl.Float64]))
+    shape: (3, 1)
+    ┌──────┐
+    │ ba   │
+    │ ---  │
+    │ str  │
+    ╞══════╡
+    │ a    │
+    ├╌╌╌╌╌╌┤
+    │ b    │
+    ├╌╌╌╌╌╌┤
+    │ null │
+    └──────┘
+
     """
     return col("*").exclude(columns)
 
 
-def all(name: Optional[Union[str, List["pli.Expr"]]] = None) -> "pli.Expr":
+def all(name: str | list[pli.Expr] | None = None) -> pli.Expr:
     """
     This function is two things
 
@@ -945,7 +997,6 @@ def all(name: Optional[Union[str, List["pli.Expr"]]] = None) -> "pli.Expr":
 
     Examples
     --------
-
     Sum all columns
 
     >>> df = pl.DataFrame(
@@ -971,16 +1022,14 @@ def all(name: Optional[Union[str, List["pli.Expr"]]] = None) -> "pli.Expr":
     return col(name).all()
 
 
-def groups(column: str) -> "pli.Expr":
+def groups(column: str) -> pli.Expr:
     """
     Syntactic sugar for `pl.col("foo").agg_groups()`.
     """
     return col(column).agg_groups()
 
 
-def quantile(
-    column: str, quantile: float, interpolation: str = "nearest"
-) -> "pli.Expr":
+def quantile(column: str, quantile: float, interpolation: str = "nearest") -> pli.Expr:
     """
     Syntactic sugar for `pl.col("foo").quantile(..)`.
     """
@@ -989,44 +1038,44 @@ def quantile(
 
 @overload
 def arange(
-    low: Union[int, "pli.Expr", "pli.Series"],
-    high: Union[int, "pli.Expr", "pli.Series"],
+    low: int | pli.Expr | pli.Series,
+    high: int | pli.Expr | pli.Series,
     step: int = ...,
     *,
     eager: Literal[False],
-) -> "pli.Expr":
+) -> pli.Expr:
     ...
 
 
 @overload
 def arange(
-    low: Union[int, "pli.Expr", "pli.Series"],
-    high: Union[int, "pli.Expr", "pli.Series"],
+    low: int | pli.Expr | pli.Series,
+    high: int | pli.Expr | pli.Series,
     step: int = ...,
     *,
     eager: Literal[True],
-) -> "pli.Series":
+) -> pli.Series:
     ...
 
 
 @overload
 def arange(
-    low: Union[int, "pli.Expr", "pli.Series"],
-    high: Union[int, "pli.Expr", "pli.Series"],
+    low: int | pli.Expr | pli.Series,
+    high: int | pli.Expr | pli.Series,
     step: int = ...,
     *,
     eager: bool = False,
-) -> Union["pli.Expr", "pli.Series"]:
+) -> pli.Expr | pli.Series:
     ...
 
 
 def arange(
-    low: Union[int, "pli.Expr", "pli.Series"],
-    high: Union[int, "pli.Expr", "pli.Series"],
+    low: int | pli.Expr | pli.Series,
+    high: int | pli.Expr | pli.Series,
     step: int = 1,
     *,
     eager: bool = False,
-) -> Union["pli.Expr", "pli.Series"]:
+) -> pli.Expr | pli.Series:
     """
     Create a range expression. This can be used in a `select`, `with_column` etc.
     Be sure that the range size is equal to the DataFrame you are collecting.
@@ -1042,9 +1091,9 @@ def arange(
     high
         Upper bound of range.
     step
-        Step size of the range
+        Step size of the range.
     eager
-        If eager evaluation is `True`, a Series is returned instead of an Expr
+        If eager evaluation is `True`, a Series is returned instead of an Expr.
     """
     low = pli.expr_to_lit_or_expr(low, str_to_lit=False)
     high = pli.expr_to_lit_or_expr(high, str_to_lit=False)
@@ -1057,9 +1106,9 @@ def arange(
 
 
 def argsort_by(
-    exprs: Union[Union["pli.Expr", str], Sequence[Union["pli.Expr", str]]],
-    reverse: Union[List[bool], bool] = False,
-) -> "pli.Expr":
+    exprs: pli.Expr | str | Sequence[pli.Expr | str],
+    reverse: list[bool] | bool = False,
+) -> pli.Expr:
     """
     Find the indexes that would sort the columns.
 
@@ -1083,14 +1132,15 @@ def argsort_by(
 
 
 def duration(
-    days: Optional[Union["pli.Expr", str]] = None,
-    seconds: Optional[Union["pli.Expr", str]] = None,
-    nanoseconds: Optional[Union["pli.Expr", str]] = None,
-    milliseconds: Optional[Union["pli.Expr", str]] = None,
-    minutes: Optional[Union["pli.Expr", str]] = None,
-    hours: Optional[Union["pli.Expr", str]] = None,
-    weeks: Optional[Union["pli.Expr", str]] = None,
-) -> "pli.Expr":
+    *,
+    days: pli.Expr | str | None = None,
+    seconds: pli.Expr | str | None = None,
+    nanoseconds: pli.Expr | str | None = None,
+    milliseconds: pli.Expr | str | None = None,
+    minutes: pli.Expr | str | None = None,
+    hours: pli.Expr | str | None = None,
+    weeks: pli.Expr | str | None = None,
+) -> pli.Expr:
     """
     Create polars `Duration` from distinct time components.
 
@@ -1117,7 +1167,7 @@ def duration(
     ...         ),
     ...         (pl.col("datetime") + pl.duration(hours="add")).alias("add_hours"),
     ...     ]
-    ... )
+    ... )  # doctest: +IGNORE_RESULT
     shape: (2, 5)
     ┌────────────┬────────────┬─────────────────────┬──────────────┬─────────────────────┐
     │ add_weeks  ┆ add_days   ┆ add_seconds         ┆ add_millisec ┆ add_hours           │
@@ -1153,14 +1203,14 @@ def duration(
 
 
 def _datetime(
-    year: Union["pli.Expr", str],
-    month: Union["pli.Expr", str],
-    day: Union["pli.Expr", str],
-    hour: Optional[Union["pli.Expr", str]] = None,
-    minute: Optional[Union["pli.Expr", str]] = None,
-    second: Optional[Union["pli.Expr", str]] = None,
-    millisecond: Optional[Union["pli.Expr", str]] = None,
-) -> "pli.Expr":
+    year: pli.Expr | str,
+    month: pli.Expr | str,
+    day: pli.Expr | str,
+    hour: pli.Expr | str | None = None,
+    minute: pli.Expr | str | None = None,
+    second: pli.Expr | str | None = None,
+    millisecond: pli.Expr | str | None = None,
+) -> pli.Expr:
     """
     Create polars `Datetime` from distinct time components.
 
@@ -1212,10 +1262,10 @@ def _datetime(
 
 
 def _date(
-    year: Union["pli.Expr", str],
-    month: Union["pli.Expr", str],
-    day: Union["pli.Expr", str],
-) -> "pli.Expr":
+    year: pli.Expr | str,
+    month: pli.Expr | str,
+    day: pli.Expr | str,
+) -> pli.Expr:
     """
     Create polars Date from distinct time components.
 
@@ -1235,24 +1285,57 @@ def _date(
     return _datetime(year, month, day).cast(Date).alias("date")
 
 
-def concat_str(
-    exprs: Union[Sequence[Union["pli.Expr", str]], "pli.Expr"], sep: str = ""
-) -> "pli.Expr":
+def concat_str(exprs: Sequence[pli.Expr | str] | pli.Expr, sep: str = "") -> pli.Expr:
     """
-    Horizontally Concat Utf8 Series in linear time. Non utf8 columns are cast to utf8.
+    Horizontally concat Utf8 Series in linear time. Non-Utf8 columns are cast to Utf8.
 
     Parameters
     ----------
     exprs
-        Columns to concat into a Utf8 Series
+        Columns to concat into a Utf8 Series.
     sep
         String value that will be used to separate the values.
+
+    Examples
+    --------
+    >>> df = pl.DataFrame(
+    ...     {
+    ...         "a": [1, 2, 3],
+    ...         "b": ["dogs", "cats", None],
+    ...         "c": ["play", "swim", "walk"],
+    ...     }
+    ... )
+    >>> df.with_columns(
+    ...     [
+    ...         pl.concat_str(
+    ...             [
+    ...                 pl.col("a") * 2,
+    ...                 pl.col("b"),
+    ...                 pl.col("c"),
+    ...             ],
+    ...             sep=" ",
+    ...         ).alias("full_sentence"),
+    ...     ]
+    ... )
+    shape: (3, 4)
+    ┌─────┬──────┬──────┬───────────────┐
+    │ a   ┆ b    ┆ c    ┆ full_sentence │
+    │ --- ┆ ---  ┆ ---  ┆ ---           │
+    │ i64 ┆ str  ┆ str  ┆ str           │
+    ╞═════╪══════╪══════╪═══════════════╡
+    │ 1   ┆ dogs ┆ play ┆ 2 dogs play   │
+    ├╌╌╌╌╌┼╌╌╌╌╌╌┼╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+    │ 2   ┆ cats ┆ swim ┆ 4 cats swim   │
+    ├╌╌╌╌╌┼╌╌╌╌╌╌┼╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+    │ 3   ┆ null ┆ walk ┆ null          │
+    └─────┴──────┴──────┴───────────────┘
+
     """
     exprs = pli.selection_to_pyexpr_list(exprs)
     return pli.wrap_expr(_concat_str(exprs, sep))
 
 
-def format(fstring: str, *args: Union["pli.Expr", str]) -> "pli.Expr":
+def format(fstring: str, *args: pli.Expr | str) -> pli.Expr:
     """
     String format utility for expressions
 
@@ -1266,7 +1349,6 @@ def format(fstring: str, *args: Union["pli.Expr", str]) -> "pli.Expr":
 
     Examples
     --------
-
     >>> df = pl.DataFrame(
     ...     {
     ...         "a": ["a", "b", "c"],
@@ -1309,9 +1391,7 @@ def format(fstring: str, *args: Union["pli.Expr", str]) -> "pli.Expr":
     return concat_str(exprs, sep="")
 
 
-def concat_list(
-    exprs: Union[Sequence[Union[str, "pli.Expr", "pli.Series"]], "pli.Expr"]
-) -> "pli.Expr":
+def concat_list(exprs: Sequence[str | pli.Expr | pli.Series] | pli.Expr) -> pli.Expr:
     """
     Concat the arrays in a Series dtype List in linear time.
 
@@ -1322,7 +1402,6 @@ def concat_list(
 
     Examples
     --------
-
     Create lagged columns and collect them into a list. This mimics a rolling window.
 
     >>> df = pl.DataFrame(
@@ -1342,21 +1421,21 @@ def concat_list(
     ...     )
     ... )
     shape: (5, 1)
-    ┌─────────────────┐
-    │ A_rolling       │
-    │ ---             │
-    │ list [f64]      │
-    ╞═════════════════╡
-    │ [null, null, 1] │
-    ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
-    │ [null, 1, 2]    │
-    ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
-    │ [1, 2, 9]       │
-    ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
-    │ [2, 9, 2]       │
-    ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
-    │ [9, 2, 13]      │
-    └─────────────────┘
+    ┌───────────────────┐
+    │ A_rolling         │
+    │ ---               │
+    │ list[f64]         │
+    ╞═══════════════════╡
+    │ [null, null, 1.0] │
+    ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+    │ [null, 1.0, 2.0]  │
+    ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+    │ [1.0, 2.0, 9.0]   │
+    ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+    │ [2.0, 9.0, 2.0]   │
+    ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+    │ [9.0, 2.0, 13.0]  │
+    └───────────────────┘
 
     """
     exprs = pli.selection_to_pyexpr_list(exprs)
@@ -1364,7 +1443,7 @@ def concat_list(
 
 
 def collect_all(
-    lazy_frames: "List[pli.LazyFrame]",
+    lazy_frames: list[pli.LazyFrame],
     type_coercion: bool = True,
     predicate_pushdown: bool = True,
     projection_pushdown: bool = True,
@@ -1372,7 +1451,7 @@ def collect_all(
     string_cache: bool = False,
     no_optimization: bool = False,
     slice_pushdown: bool = False,
-) -> "List[pli.DataFrame]":
+) -> list[pli.DataFrame]:
     """
     Collect multiple LazyFrames at the same time. This runs all the computation graphs in parallel on
     Polars threadpool.
@@ -1430,8 +1509,8 @@ def collect_all(
 
 
 def select(
-    exprs: Union[str, "pli.Expr", Sequence[str], Sequence["pli.Expr"], "pli.Series"]
-) -> "pli.DataFrame":
+    exprs: str | pli.Expr | Sequence[str | pli.Expr] | pli.Series,
+) -> pli.DataFrame:
     """
     Run polars expressions without a context.
 
@@ -1441,13 +1520,13 @@ def select(
     ----------
     exprs
         Expressions to run
+
     Returns
     -------
     DataFrame
 
     Examples
     --------
-
     >>> foo = pl.Series("foo", [1, 2, 3])
     >>> bar = pl.Series("bar", [3, 2, 1])
     >>> pl.select(
@@ -1474,40 +1553,32 @@ def select(
 
 @overload
 def struct(
-    exprs: Union[
-        Sequence[Union["pli.Expr", str, "pli.Series"]], "pli.Expr", "pli.Series"
-    ],
+    exprs: Sequence[pli.Expr | str | pli.Series] | pli.Expr | pli.Series,
     eager: Literal[True],
-) -> "pli.Series":
+) -> pli.Series:
     ...
 
 
 @overload
 def struct(
-    exprs: Union[
-        Sequence[Union["pli.Expr", str, "pli.Series"]], "pli.Expr", "pli.Series"
-    ],
+    exprs: Sequence[pli.Expr | str | pli.Series] | pli.Expr | pli.Series,
     eager: Literal[False],
-) -> "pli.Expr":
+) -> pli.Expr:
     ...
 
 
 @overload
 def struct(
-    exprs: Union[
-        Sequence[Union["pli.Expr", str, "pli.Series"]], "pli.Expr", "pli.Series"
-    ],
+    exprs: Sequence[pli.Expr | str | pli.Series] | pli.Expr | pli.Series,
     eager: bool = False,
-) -> Union["pli.Expr", "pli.Series"]:
+) -> pli.Expr | pli.Series:
     ...
 
 
 def struct(
-    exprs: Union[
-        Sequence[Union["pli.Expr", str, "pli.Series"]], "pli.Expr", "pli.Series"
-    ],
+    exprs: Sequence[pli.Expr | str | pli.Series] | pli.Expr | pli.Series,
     eager: bool = False,
-) -> Union["pli.Expr", "pli.Series"]:
+) -> pli.Expr | pli.Series:
     """
     Collect several columns into a Series of dtype Struct
 
@@ -1520,7 +1591,6 @@ def struct(
 
     Examples
     --------
-
     >>> pl.DataFrame(
     ...     {
     ...         "int": [1, 2],
@@ -1530,15 +1600,15 @@ def struct(
     ...     }
     ... ).select([pl.struct(pl.all()).alias("my_struct")])
     shape: (2, 1)
-    ┌───────────────────────┐
-    │ my_struct             │
-    │ ---                   │
-    │ struct{int, ... list} │
-    ╞═══════════════════════╡
-    │ {1,"a",true,[1, 2]}   │
-    ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
-    │ {2,"b",null,[3]}      │
-    └───────────────────────┘
+    ┌─────────────────────┐
+    │ my_struct           │
+    │ ---                 │
+    │ struct[4]           │
+    ╞═════════════════════╡
+    │ {1,"a",true,[1, 2]} │
+    ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+    │ {2,"b",null,[3]}    │
+    └─────────────────────┘
 
     Only collect specific columns as a struct:
 
@@ -1547,19 +1617,19 @@ def struct(
     ... )
     >>> df.with_column(pl.struct(pl.col(["a", "b"])).alias("a_and_b"))
     shape: (4, 4)
-    ┌─────┬───────┬─────┬───────────────────────────────┐
-    │ a   ┆ b     ┆ c   ┆ a_and_b                       │
-    │ --- ┆ ---   ┆ --- ┆ ---                           │
-    │ i64 ┆ str   ┆ i64 ┆ struct[2]{'a': i64, 'b': str} │
-    ╞═════╪═══════╪═════╪═══════════════════════════════╡
-    │ 1   ┆ one   ┆ 9   ┆ {1,"one"}                     │
-    ├╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
-    │ 2   ┆ two   ┆ 8   ┆ {2,"two"}                     │
-    ├╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
-    │ 3   ┆ three ┆ 7   ┆ {3,"three"}                   │
-    ├╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
-    │ 4   ┆ four  ┆ 6   ┆ {4,"four"}                    │
-    └─────┴───────┴─────┴───────────────────────────────┘
+    ┌─────┬───────┬─────┬─────────────┐
+    │ a   ┆ b     ┆ c   ┆ a_and_b     │
+    │ --- ┆ ---   ┆ --- ┆ ---         │
+    │ i64 ┆ str   ┆ i64 ┆ struct[2]   │
+    ╞═════╪═══════╪═════╪═════════════╡
+    │ 1   ┆ one   ┆ 9   ┆ {1,"one"}   │
+    ├╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+    │ 2   ┆ two   ┆ 8   ┆ {2,"two"}   │
+    ├╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+    │ 3   ┆ three ┆ 7   ┆ {3,"three"} │
+    ├╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+    │ 4   ┆ four  ┆ 6   ┆ {4,"four"}  │
+    └─────┴───────┴─────┴─────────────┘
 
     """
 
@@ -1571,44 +1641,44 @@ def struct(
 
 @overload
 def repeat(
-    value: Optional[Union[float, int, str, bool]],
-    n: Union["pli.Expr", int],
+    value: float | int | str | bool | None,
+    n: pli.Expr | int,
     *,
     eager: Literal[False] = ...,
-    name: Optional[str] = ...,
-) -> "pli.Expr":
+    name: str | None = ...,
+) -> pli.Expr:
     ...
 
 
 @overload
 def repeat(
-    value: Optional[Union[float, int, str, bool]],
-    n: Union["pli.Expr", int],
+    value: float | int | str | bool | None,
+    n: pli.Expr | int,
     *,
     eager: Literal[True],
-    name: Optional[str] = ...,
-) -> "pli.Series":
+    name: str | None = ...,
+) -> pli.Series:
     ...
 
 
 @overload
 def repeat(
-    value: Optional[Union[float, int, str, bool]],
-    n: Union["pli.Expr", int],
+    value: float | int | str | bool | None,
+    n: pli.Expr | int,
     *,
     eager: bool,
-    name: Optional[str],
-) -> Union["pli.Expr", "pli.Series"]:
+    name: str | None,
+) -> pli.Expr | pli.Series:
     ...
 
 
 def repeat(
-    value: Optional[Union[float, int, str, bool]],
-    n: Union["pli.Expr", int],
+    value: float | int | str | bool | None,
+    n: pli.Expr | int,
     *,
     eager: bool = False,
-    name: Optional[str] = None,
-) -> Union["pli.Expr", "pli.Series"]:
+    name: str | None = None,
+) -> pli.Expr | pli.Series:
     """
     Repeat a single value n times.
 
@@ -1633,3 +1703,60 @@ def repeat(
         if isinstance(n, int):
             n = lit(n)
         return pli.wrap_expr(_repeat(value, n._pyexpr))
+
+
+@overload
+def arg_where(
+    condition: pli.Expr | pli.Series,
+    eager: Literal[False] = ...,
+) -> pli.Expr:
+    ...
+
+
+@overload
+def arg_where(condition: pli.Expr | pli.Series, eager: Literal[True]) -> pli.Series:
+    ...
+
+
+@overload
+def arg_where(condition: pli.Expr | pli.Series, eager: bool) -> pli.Expr | pli.Series:
+    ...
+
+
+def arg_where(
+    condition: pli.Expr | pli.Series, eager: bool = False
+) -> pli.Expr | pli.Series:
+    """
+    Return indices where `condition` evaluates `True`.
+
+    Parameters
+    ----------
+    condition
+        Boolean expression to evaluate
+
+    Examples
+    --------
+    >>> df = pl.DataFrame({"a": [1, 2, 3, 4, 5]})
+    >>> df.select(
+    ...     [
+    ...         pl.arg_where(pl.col("a") % 2 == 0),
+    ...     ]
+    ... ).to_series()
+    shape: (2,)
+    Series: 'a' [u32]
+    [
+        1
+        3
+    ]
+    """
+    if eager:
+        if not isinstance(condition, pli.Series):
+            raise ValueError(
+                f"expected 'Series' in 'arg_where' if 'eager=True', got {type(condition)}"
+            )
+        return (
+            condition.to_frame().select(arg_where(pli.col(condition.name))).to_series()
+        )
+    else:
+        condition = pli.expr_to_lit_or_expr(condition, str_to_lit=True)
+        return pli.wrap_expr(py_arg_where(condition._pyexpr))
