@@ -1,8 +1,6 @@
-# flake8: noqa: W191,E101
 from __future__ import annotations
 
 import os
-from typing import List
 
 import pytest
 
