@@ -1,11 +1,8 @@
-# flake8: noqa: W191,E101
 from __future__ import annotations
 
 import io
 import os
 
-import pytest
-
 import polars as pl
 
 
