@@ -1,4 +1,3 @@
-# flake8: noqa: W191,E101
 from __future__ import annotations
 
 import sys
@@ -6,7 +5,7 @@ import typing
 from builtins import range
 from datetime import date, datetime, timedelta
 from io import BytesIO
-from typing import Any, Iterator, Type
+from typing import Any, Iterator
 from unittest.mock import patch
 
 import numpy as np
@@ -20,11 +19,11 @@ from polars.testing import assert_frame_equal, assert_series_equal, columns
 if sys.version_info >= (3, 8):
     from typing import Literal
 else:
-    from typing_extensions import Literal  # pragma: no cover
+    from typing_extensions import Literal
 
 
 def test_version() -> None:
-    _version = pl.__version__
+    pl.__version__
 
 
 def test_null_count() -> None:
@@ -182,7 +181,7 @@ def test_init_ndarray() -> None:
     assert df.frame_equal(truth)
 
     # 2D array - default to column orientation
-    df = pl.DataFrame(np.array([[1, 2], [3, 4]]), orient="column")
+    df = pl.DataFrame(np.array([[1, 2], [3, 4]]), orient="col")
     truth = pl.DataFrame({"column_0": [1, 2], "column_1": [3, 4]})
     assert df.frame_equal(truth)
 
@@ -509,6 +508,13 @@ def test_from_arrow() -> None:
     assert pl.from_arrow(tbl).shape == (2, 5)
 
 
+def test_dataframe_membership_operator() -> None:
+    # cf. issue #4032
+    df = pl.DataFrame({"name": ["Jane", "John"], "age": [20, 30]})
+    assert "name" in df
+    assert "phone" not in df
+
+
 def test_sort() -> None:
     df = pl.DataFrame({"a": [2, 1, 3], "b": [1, 2, 3]})
     with pytest.deprecated_call():
@@ -738,39 +744,6 @@ def test_groupby() -> None:
     assert df.groupby("b").agg_list().shape == (2, 3)
 
 
-def test_pivot() -> None:
-    df = pl.DataFrame(
-        {
-            "a": [1, 2, 3, 4, 5],
-            "b": ["a", "a", "b", "b", "b"],
-            "c": [None, 1, None, 1, None],
-        }
-    )
-    gb = df.groupby("b").pivot("a", "c")
-    assert gb.first().shape == (2, 6)
-    assert gb.max().shape == (2, 6)
-    assert gb.mean().shape == (2, 6)
-    assert gb.count().shape == (2, 6)
-    assert gb.median().shape == (2, 6)
-
-    for agg_fn in ["sum", "min", "max", "mean", "count", "median", "mean"]:
-        out = df.pivot(
-            values="c", index="b", columns="a", aggregate_fn=agg_fn, sort_columns=True
-        )
-        assert out.shape == (2, 6)
-
-    # example in polars-book
-    df = pl.DataFrame(
-        {
-            "foo": ["A", "A", "B", "B", "C"],
-            "N": [1, 2, 2, 4, 2],
-            "bar": ["k", "l", "m", "n", "o"],
-        }
-    )
-    out = df.groupby("foo").pivot(pivot_column="bar", values_column="N").first()
-    assert out.shape == (3, 6)
-
-
 def test_join() -> None:
     df_left = pl.DataFrame(
         {
@@ -1502,6 +1475,33 @@ def test_hash_rows() -> None:
     assert df.select([pl.col("a").hash().alias("foo")])["foo"].dtype == pl.UInt64
 
 
+def test_reproducible_hash_with_seeds() -> None:
+    """Tests the reproducibility of DataFrame.hash_rows, Series.hash, and Expr.hash.
+
+    cf. issue #3966, hashes must always be reproducible across sessions when using
+    the same seeds.
+
+    """
+    df = pl.DataFrame({"s": [1234, None, 5678]})
+    seeds = (11, 22, 33, 44)
+    expected = pl.Series(
+        "s",
+        [
+            15801072432137883943,
+            988796329533502010,
+            9604537446374444741,
+        ],
+        dtype=pl.UInt64,
+    )
+
+    result = df.hash_rows(*seeds)
+    assert_series_equal(expected, result, check_names=False, check_exact=True)
+    result = df["s"].hash(*seeds)
+    assert_series_equal(expected, result, check_names=False, check_exact=True)
+    result = df.select([pl.col("s").hash(*seeds)])["s"]
+    assert_series_equal(expected, result, check_names=False, check_exact=True)
+
+
 def test_create_df_from_object() -> None:
     class Foo:
         def __init__(self, value: int) -> None:
@@ -1615,7 +1615,7 @@ def test_apply_dataframe_return() -> None:
     assert out.frame_equal(expected, null_equal=True)
 
 
-def test_groupby_cat_list() -> None:  # noqa: W191,E101
+def test_groupby_cat_list() -> None:
     grouped = (
         pl.DataFrame(
             [
@@ -2053,23 +2053,6 @@ def test_get_item() -> None:
         _ = df[pl.Series("", ["hello Im a string"])]
 
 
-def test_pivot_list() -> None:
-    df = pl.DataFrame({"a": [1, 2, 3], "b": [[1, 1], [2, 2], [3, 3]]})
-
-    expected = pl.DataFrame(
-        {
-            "a": [1, 2, 3],
-            "1": [[1, 1], None, None],
-            "2": [None, [2, 2], None],
-            "3": [None, None, [3, 3]],
-        }
-    )
-
-    out = df.pivot("b", index="a", columns="a", aggregate_fn="first", sort_columns=True)
-
-    assert out.frame_equal(expected, null_equal=True)
-
-
 @pytest.mark.parametrize("as_series,inner_dtype", [(True, pl.Series), (False, list)])
 def test_to_dict(as_series: bool, inner_dtype: Any) -> None:
     df = pl.DataFrame(
@@ -2145,9 +2128,9 @@ def test_join_suffixes() -> None:
 
     for how in ["left", "inner", "outer", "cross"]:
         # no need for an assert, we error if wrong
-        _df = df_a.join(df_b, on="A", suffix="_y", how=how)["B_y"]
+        df_a.join(df_b, on="A", suffix="_y", how=how)["B_y"]
 
-    _df = df_a.join_asof(df_b, on="A", suffix="_y")["B_y"]
+    df_a.join_asof(df_b, on="A", suffix="_y")["B_y"]
 
 
 def test_preservation_of_subclasses() -> None:
