@@ -24,3 +24,10 @@ def test_schema_on_agg() -> None:
         "first": pl.Int64,
         "last": pl.Int64,
     }
+
+
+def test_fill_null_minimal_upcast_4056() -> None:
+    df = pl.DataFrame({"a": [-1, 2, None]})
+    df = df.with_columns(pl.col("a").cast(pl.Int8))
+    assert df.with_column(pl.col(pl.Int8).fill_null(-1)).dtypes[0] == pl.Int8
+    assert df.with_column(pl.col(pl.Int8).fill_null(-1000)).dtypes[0] == pl.Int32
